@@ -1,4 +1,5 @@
 #define QT_FEATURE_clock_gettime 1
+#define QT_FEATURE_dlopen 1
 #define QT_FEATURE_doubleconversion 1
 #define QT_FEATURE_glib 1
 #define QT_FEATURE_posix_libiconv 1
@@ -15,5 +16,5 @@
 #define QT_FEATURE_sha3_fast 1
 #define QT_FEATURE_slog2 -1
 #define QT_FEATURE_syslog -1
-#define QT_FEATURE_system_doubleconversion -1
-#define QT_FEATURE_system_pcre 1
+#define QT_FEATURE_system_doubleconversion 1
+#define QT_FEATURE_system_pcre2 1
